(** * Interface calculatrice *)

open Graphics ;;
open Parser ;;
open Calculator ;;

(** ** Type definition *)
type button = int*int*string ;;

(** ** New color *)
let grey = rgb 150 150 150 ;;

(** ** I. Interface design *)

(** *** Draw button *)
let draw_button (x_coord, y_coord, x_size, y_size, text) =
  set_color grey ;
  fill_rect x_coord y_coord x_size y_size ;
  set_color black ;
  let (x_t, y_t) = text_size text in
  moveto (x_coord + (x_size - x_t)/2) (y_coord + (y_size - y_t)/2) ;
  draw_string text ;;

(** *** Draw the calculator *)
let init () =
  (* Opening window *)
  open_graph "" ;
  clear_graph () ;
  (* resizing *)
  resize_window 200 300 ;
  (* Background *)
  set_color black ;
  fill_rect 0 0 200 300 ;
  (* Screen *)
  set_color grey ;
  fill_rect 30 240 140 30 ;
  (* Buttons *)
  let label_button = function
    | (0,0) -> "0"
    | (1,0) -> "."
    | (2,0) -> "C"
    | (3,0) -> "+"
    | (0,1) -> "1"
    | (1,1) -> "2"
    | (2,1) -> "3"
    | (3,1) -> "-"
    | (0,2) -> "4"
    | (1,2) -> "5"
    | (2,2) -> "6"
    | (3,2) -> "*"
    | (0,3) -> "7"
    | (1,3) -> "8"
    | (2,3) -> "9"
    | (3,3) -> "/"
    | _ -> ""
  in
  let buttons = ref [] in
  for i = 0 to 3 do
    for j = 0 to 3 do
      buttons := (30+38*i, 40+38*j, 26, 26, label_button (i,j)) :: !buttons
    done
  done;
  (* Special case for '=' *)
  buttons := (106,2,62, 26, "=") :: !buttons ;
  List.iter draw_button !buttons ;
  !buttons ;;


(** *** Updates the screen *)
let update_screen s =
  set_color grey ;
  fill_rect 30 240 140 30 ;
  set_color black ;
  moveto 36 248 ;
  draw_string s ;;


(** ** II. User interactions *)


let allowed_key = ['0'; '1'; '2'; '3'; '4'; '5'; '6'; '7'; '8'; '9'; '+'; '-'; '*'; '/'; '.'] ;;
let string_of_char k = String.make 1 k ;;

(** *** When the user press a key *)
let key_update key screen =
  match key with
  | k when List.mem k allowed_key -> screen := !screen ^ (string_of_char k)
  | '=' ->
     begin
       try
         screen := string_of_int (compute (parse !screen))
       with Parsing_error -> screen := "ERROR"
     end
  | 'C' -> screen := "" ;
  | '\b' ->
     let len = String.length !screen in
     if len > 0 then
       screen := String.sub !screen 0 (len - 1)
  (* Anti-Warning *)
  | _ ->  () ;;

(** *** Searches if the mouse clicked on some button *)
let rec on_button mouse_x mouse_y = function
  | [] -> raise Not_found
  | (x_coord, y_coord, x_size, y_size, text) :: buttons ->
     if mouse_x < x_coord + x_size &&
          mouse_x > x_coord &&
            mouse_y < y_coord + y_size &&
              mouse_y > y_coord
     then text
     else on_button mouse_x mouse_y buttons ;;

(** *** When the user click on a button *)
let mouse_update buttons mouse_x mouse_y screen =
  try
    key_update ((on_button mouse_x mouse_y buttons).[0]) screen
  with Not_found -> () ;;

(** ** Main function *)
let main () =
  let buttons = init () in
  let screen = ref "" in
  while true do
    let status = wait_next_event [Button_down; Key_pressed] in
    begin
      match status.keypressed with
      | true -> key_update status.key screen
      | false -> mouse_update buttons status.mouse_x status.mouse_y screen ;
    end ;
    update_screen !screen
  done ;;

main () ;;
